package no.hig.okolloen.graphics;

import java.awt.*;
import java.awt.event.*;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.util.Vector;

/**
 * Knob is the basis if other dialknob classes and provides the display
 * of a basic knob and interaction with the mouse.
 * You can set various options for the Knob component like :
 * <ul>
 * <li>The max and min angle used as the active area
 * <li>The max and min values returned by this component
 * <li>The color of the top of the knob
 * </ul>
 * <p>
 * The knob component is freely scalable in both directions.
 * 
 * @author      Oivind Kolloen
 * @version     %I%, %G%
 * @since       rev 0.1
 */
public class Knob extends Panel implements MouseListener, MouseMotionListener, Adjustable {
    protected transient Graphics bg;
    protected transient Image buffer;
    protected transient int bufferWidth; 
    protected transient int bufferHeight;
    private Color color = new Color (198, 198, 198);
    private double minValue = 0;
    private double maxValue = 100;
    protected int minAngle = -45;
    protected int maxAngle = 225;
    private double currentValue = -1;
    protected int knobxoffset;
    protected int knobyoffset;
    protected int size;
    private Vector adjustmentListeners = new Vector ();

    /**
     *	Default constructor for Dial class.
     */
    public Knob () {
        super ();
        knobxoffset = 2;
        knobyoffset = 2;
        size = 20;
        addMouseListener (this);
        addMouseMotionListener (this);
    }

    /**
     *  Sets the current value.
     *  @param current the new current value.
     */
    public void setValue (double value) {
        currentValue = value;
        repaint ();
    }

    /**
     *  Gets the value representing the state of this component.
     *  @return double value representing the state of this component.
     */
    public double getDoubleValue () {
        return maxValue-currentValue;
    }

    /**
     *  Sets the minimum value allowed.
     *  @param min the new minimum value.
     */
    public void setMinValue (double min) {
        minValue = min;
        setValue (currentValue);
    }

    /**
     *  Sets the maximum value allowed.
     *  @param max the new maximum value.
     */
    public void setMaxValue (double max) {
        maxValue = max;
        setValue (currentValue);
    }

    /**
     *  Sets the minimum angle used.
     *  @param min the new minimum angle.
     */
    public void setMinAngle (int min) {
        minAngle = min;
        setValue (currentValue);
    }

    /**
     *  Sets the maximum angle used.
     *  @param max the new maximum angle.
     */
    public void setMaxAngle (int max) {
        maxAngle = max;
        setValue (currentValue);
    }

    /**
     *  Sets the color of the top part of the knob.
     *  @color the new color to use when drawing the top of the knob.
     */
    public void setColor (Color color) {
        this.color = color;
    }

    /**
     *	Updates this component.
     *      @param g the graphics context.
     *	@see java.awt.Component#update
     */
    public void update(Graphics g) {
        paint(g);
    }

    /**
     *	Paints this component.
     *	@param g the graphics context.
     *	@see java.awt.Component#paint
     */	
    public void paint(Graphics g) {
        int height = getSize().height;
        int width = getSize().width;

        if(buffer == null || width != bufferWidth || height != bufferHeight) {
            buffer = createImage(width, height);
            bg = buffer.getGraphics();
            bufferWidth = width;
            bufferHeight = height;
        }

        bg.setColor (getBackground());
        bg.fillRect (0, 0, width, height);
        bg.setColor (new Color (132, 132, 132));
        bg.fillOval (knobxoffset, knobyoffset, size, size);
        bg.setColor (Color.white);
        bg.fillOval (knobxoffset+2, knobyoffset+2, size-4, size-4);
        bg.setColor (color);
        bg.fillOval (knobxoffset+4, knobyoffset+4, size-8, size-8);
        
        if ((currentValue <= maxValue)&&(currentValue >= minValue))
            drawDial (bg, knobxoffset, knobyoffset, size, size);
        Rectangle clip = g.getClipBounds();
        if(clip != null)
            g.setClip(clip);

        g.drawImage(buffer, 0, 0, this);
    }

    /**
     *  Returns the preferred size of this component.
     *  @return A Dimension object representing the preferred size of this component.
     *  @see java.awt.Component#getPreferredSize
     */
    public Dimension getPreferredSize() {
        return new Dimension(size+knobxoffset*2, size+knobyoffset*2);
    }

    /**
     *  Gets the orientation of this component, will always return DragControl.VERTICAL.
     */
    public int getOrientation() {
        return VERTICAL;
    }

    /**
     *  Sets the minimum value of the DragControl object.
     *  @param min the minimum value
     */
    public void setMinimum(int min) {
        setMinValue ((double)min);
    }

    /**
     *  Gets the minimum value of the DragControl object. (truncated)
     */
    public int getMinimum() {
        return (int)minValue;
    }
    
    /**
     *  Sets the maximum value of the DragControl object.
     *  @param min the minimum value
     */
    public void setMaximum(int max) {
        setMaxValue ((double)max);
    }

    /**
     *  Gets the maximum value of the DragControl object. (truncated)
     */
    public int getMaximum() {
        return (int)maxValue;
    }

    /**
     *  Dummy to satisfy the Adjustable interface
     */
    public void setUnitIncrement(int u) {
    }

    /**
     *  Dummy to satisfy the Adjustable interface, always returns 1
     */
    public int getUnitIncrement() {
        return 1;
    }

    /**
     *  Dummy to satisfy the Adjustable interface
     */
    public void setBlockIncrement(int b) {
    }

    /**
     *  Dummy to satisfy the Adjustable interface, always returns 1
     */
    public int getBlockIncrement() {
        return 1;
    }

    /**
     *  Dummy to satisfy the Adjustable interface
     */
    public void setVisibleAmount(int v) {
    }

    /**
     *  Dummy to satisfy the Adjustable interface, always returns 1
     */
    public int getVisibleAmount() {
        return 1;
    }

    /**
     *  Sets the current value of the DragControl object.
     *  @param v the new value
     */
    public void setValue(int v) {
        setValue ((double)v);
    }

    /**
     *  Gets the current value of the DragControl object. (truncated)
     */
    public int getValue() {
        return (int)(maxValue-currentValue);
    }

    /**
     *  Add a listener to recieve adjustment events when the value of 
     *  the adjustable object changes.
     *  @param l the listener to recieve events
     *	@see java.awt.event.AdjustmentEvent
     */
    public void addAdjustmentListener (AdjustmentListener l) {
        adjustmentListeners.add (l);
    }

    /**
     *  Removes an adjustment listener.
     *  @param l the listener being removed
     *	@see java.awt.event.AdjustmentEvent
     */
    public void removeAdjustmentListener(AdjustmentListener l) {
        adjustmentListeners.removeElement (l);
    }

/********************************************************************************/
/* Only private and helper functions below this point,                          */
/* nothing you will want to mess with.                                          */
/********************************************************************************/

    public void mouseClicked(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
        updatePos(e);
    }

    public void mouseReleased(MouseEvent e) {
        updatePos(e);
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mouseDragged(MouseEvent e) {
        updatePos(e);
    }

    public void mouseMoved(MouseEvent e) {
    }

    void updatePos(MouseEvent e) {
        int x = e.getX();
        int y = e.getY();
        
        double newx = (x-knobxoffset)-(double)size/2;
        double newy = (y-knobyoffset)-(double)size/2;
        newy *= -1;
        double longest = java.lang.Math.sqrt(newx*newx+newy*newy);
        double newAngle = java.lang.Math.acos (newx/longest);
        newAngle = java.lang.Math.toDegrees (newAngle);
        if ((newAngle>=90)&&(newy<0))
            newAngle = 360-newAngle;
        if ((newAngle<90)&&(newy<0))
            newAngle *= -1;
        if (newAngle > maxAngle) 
            newAngle = maxAngle;
        if (newAngle < minAngle)
            newAngle = minAngle;
        double normalizedValue = (newAngle-minAngle)/(maxAngle-minAngle);
        setValue (((maxValue-minValue)*normalizedValue)+minValue);
        if (adjustmentListeners.size() > 0) {
            int i=0; 
            AdjustmentEvent ae = new AdjustmentEvent ((Adjustable)e.getComponent(), AdjustmentEvent.ADJUSTMENT_VALUE_CHANGED, AdjustmentEvent.TRACK, (int)(maxValue-currentValue));
            while (i<adjustmentListeners.size())
                ((AdjustmentListener)adjustmentListeners.elementAt(i++)).adjustmentValueChanged (ae);
        }
    }

    private void drawDial (Graphics g, int xoffset, int yoffset, int height, int width) {
        g.setColor (Color.black);
        int angleSpan = maxAngle-minAngle;
        if (currentValue < minValue) 
            currentValue = minValue;
        if (currentValue > maxValue) 
            currentValue = maxValue;
        double valueSpan = maxValue-minValue;
        double currentNormalizedValue = currentValue/valueSpan;
        double angleToDisplay = angleSpan*currentNormalizedValue;
        angleToDisplay += minAngle;
        
        double xOffs = (double)xoffset+(double)width/(double)2;
        double xAngleValue = java.lang.Math.cos (java.lang.Math.toRadians(angleToDisplay));
        double xSize = (double)width/(double)2;
        double yOffs = (double)yoffset+(double)height/(double)2;
        double yAngleValue = java.lang.Math.sin (java.lang.Math.toRadians(angleToDisplay));
        double ySize = (double)height/(double)2;

        int tipXCoord = ((int)(xOffs+xAngleValue*xSize));
        int tipYCoord = ((int)(yOffs-yAngleValue*ySize));
       
        int startXCoord = (int)((xOffs+tipXCoord)/2);
        int startYCoord = (int)((yOffs+tipYCoord)/2);

        if ((angleToDisplay>45)&&(angleToDisplay<=135)) {
            g.drawLine (tipXCoord, tipYCoord, startXCoord, startYCoord);
            g.drawLine (tipXCoord+1, tipYCoord, startXCoord+1, startYCoord);
        } else if ((angleToDisplay>135)&&(angleToDisplay<=225)) {
            g.drawLine (tipXCoord, tipYCoord, startXCoord, startYCoord);
            g.drawLine (tipXCoord, tipYCoord+1, startXCoord, startYCoord+1);
        } else if ((angleToDisplay>225)&&(angleToDisplay<=315)) {
            g.drawLine (tipXCoord, tipYCoord, startXCoord, startYCoord);
            g.drawLine (tipXCoord-1, tipYCoord, startXCoord-1, startYCoord);
        } else if ((angleToDisplay>315)||(angleToDisplay<=45)) {
            g.drawLine (tipXCoord, tipYCoord, startXCoord, startYCoord);
            g.drawLine (tipXCoord, tipYCoord-1, startXCoord, startYCoord-1);
        }
    }
}
