package no.hig.okolloen.graphics.chart;

import java.awt.Color;

/**
 * The ScaledChartDataSet class is used to hold scaleable data values used by LineChart and BarChart.
 * It is more or less just a vector of ChartItems.
 * 
 * @author      Oivind Kolloen
 * @version     %I%, %G%
 * @since       rev 0.1
 */
public class ScaledChartDataSet extends ChartDataSet {
    ValueScaler vs = new ValueScaler (1, 0);

    /**
     *	Constructor for the ScaledChartDataSet class.
     *  @param maxElements the maximum number of elements held by this ChartDataSet.
     */
    public ScaledChartDataSet (int maxElements) {
        super (maxElements);
    }

    /**
     *  Used perform a rescale of the dataset. Use it before using the dataset
     *  in any LineChart or BarChart and when you suspect large changes in your dataset.
     */
    public void rescale () {
        double min;
        double max;
        if (dataSet.size()==0)
            return;
        ChartItem gi = getElementAt (0);
        min = gi.getValue();
        max = gi.getValue();
        for (int i=1; i<dataSet.size(); i++) {
            gi = getElementAt (i);
            if (gi.getValue()<min)
                min = gi.getValue();
            else if (gi.getValue()>max)
                max = gi.getValue();
        }
        vs.rescale (max, min);
    }

    /**
     *	Used to retrieve a normalized representation for the value in the given item.
     *  @return double value with the normalized value for the given datapoint.
     */
    public double getScaledElementAt (int element) {
        return vs.getNormalizedValue (getElementAt(element).getValue());
    }

    /**
     *	Returns the labels that should be used to indicate max, min and intermediate values.
     *  @return String[] containing five strings with max, min and intermedtiate values.
     */
    public String[] getScaleLabels () {
        return vs.getScaleLabels ();
    }

    /**
     *	Adds an element to the ChartDataSet.
     *  If this results in more than the maximum number of items being held in the 
     *  dataset the first element is thrown away. Rescaling is done if this value breaks the max/min limit.
     *  @param item the ChartItem to be added.
     */
    public void addElement (ChartItem item) {
        super.addElement (item);
        if (item.getValue()<vs.getMin())
            vs.rescale (vs.getMax(), item.getValue());
        if (item.getValue()>vs.getMax())
            vs.rescale (item.getValue(), vs.getMin());
    }

    /**
     *	Adds an element to the ChartDataSet.
     *  If this results in more than the maximum number of items being held in the 
     *  dataset the first element is thrown away. Rescaling is done if this value breaks the max/min limit.
     *  @param value the value of this datapoint.
     */
    public void addElement (double value) {
        super.addElement (value);
        if (value<vs.getMin())
            vs.rescale (vs.getMax(), value);
        if (value>vs.getMax())
            vs.rescale (value, vs.getMin());
    }

    /**
     *	Adds an element to the ChartDataSet.
     *  If this results in more than the maximum number of items being held in the 
     *  dataset the first element is thrown away.
     *  Rescaling is done if this value breaks the max/min limit.
     *  @param value the value of this datapoint.
     *  @param color the color that will be used when representing this datapoint
     */
    public void addElement (double value, Color color) {
        super.addElement (value, color);
        if (value<vs.getMin())
            vs.rescale (vs.getMax(), value);
        if (value>vs.getMax())
            vs.rescale (value, vs.getMin());
    }

    /**
     *	Adds an element to the ChartDataSet.
     *  If this results in more than the maximum number of items being held in the 
     *  dataset the first element is thrown away.
     *  Rescaling is done if this value breaks the max/min limit.
     *  @param value the value of this datapoint.
     *  @param color the color that will be used when representing this datapoint
     *  @param title the title to give this datapoint
     */
    public void addElement (double value, Color color, String title) {
        super.addElement (value, color, title);
        if (value<vs.getMin())
            vs.rescale (vs.getMax(), value);
        if (value>vs.getMax())
            vs.rescale (value, vs.getMin());
    }

    /**
     *	Sets element at given index in the ChartDataSet to given value.
     *  Rescaling is done if this value breaks the max/min limit.
     *  @param value the value of this datapoint.
     */
    public void setElementAt (int element, double value) {
        super.setElementAt (element, value);
        if (value<vs.getMin())
            vs.rescale (vs.getMax(), value);
        if (value>vs.getMax())
            vs.rescale (value, vs.getMin());
    }

    /**
     *	Sets element at given index in the ChartDataSet to given value and color.
     *  Rescaling is done if this value breaks the max/min limit.
     *  @param value the value of this datapoint.
     *  @param color the color of used for the datapoint.
     */
    public void setElementAt (int element, double value, Color color) {
        super.setElementAt (element, value, color);
        if (value<vs.getMin())
            vs.rescale (vs.getMax(), value);
        if (value>vs.getMax())
            vs.rescale (value, vs.getMin());
    }

    /**
     *	Sets element at given index in the ChartDataSet to given value, color and with given title.
     *  Rescaling is done if this value breaks the max/min limit.
     *  @param value the value of this datapoint.
     *  @param color the color of used for the datapoint.
     *  @param title the title to use for this datapoint.
     */
    public void setElementAt (int element, double value, Color color, String title) {
        super.setElementAt (element, value, color, title);
        if (value<vs.getMin())
            vs.rescale (vs.getMax(), value);
        if (value>vs.getMax())
            vs.rescale (value, vs.getMin());
    }
}